@@ -22,7 +22,7 @@ from mlflow.utils import get_unique_resource_id
 from mlflow.utils.annotations import experimental
 from mlflow.utils.file_utils import TempDir
 from mlflow.models.container import SUPPORTED_FLAVORS as SUPPORTED_DEPLOYMENT_FLAVORS
-from mlflow.models.container import DEPLOYMENT_CONFIG_KEY_FLAVOR_NAME
+from mlflow.models.container import DEPLOYMENT_CONFIG_KEY_FLAVOR_NAME, SERVING_ENVIRONMENT
 
 
 DEFAULT_IMAGE_NAME = "mlflow-pyfunc"
@@ -41,6 +41,8 @@ DEFAULT_BUCKET_NAME_PREFIX = "mlflow-sagemaker"
 DEFAULT_SAGEMAKER_INSTANCE_TYPE = "ml.m4.xlarge"
 DEFAULT_SAGEMAKER_INSTANCE_COUNT = 1
 
+SAGEMAKER_SERVING_ENVIRONMENT = "SageMaker"
+
 _logger = logging.getLogger(__name__)
 
 _full_template = "{account}.dkr.ecr.{region}.amazonaws.com/{image}:{version}"
@@ -1208,7 +1210,10 @@ def _get_deployment_config(flavor_name):
     """
     :return: The deployment configuration as a dictionary
     """
-    deployment_config = {DEPLOYMENT_CONFIG_KEY_FLAVOR_NAME: flavor_name}
+    deployment_config = {
+        DEPLOYMENT_CONFIG_KEY_FLAVOR_NAME: flavor_name,
+        SERVING_ENVIRONMENT: SAGEMAKER_SERVING_ENVIRONMENT,
+    }
     return deployment_config
 
 
@@ -1575,8 +1580,8 @@ def _update_sagemaker_endpoint(
             failure_reason = endpoint_info.get(
                 "FailureReason",
                 (
-                    "An unknown SageMaker failure occurred. \
-                    Please see the SageMaker console logs for"  # noqa
+                    "An unknown SageMaker failure occurred."
+                    " Please see the SageMaker console logs for"
                     " more information."
                 ),
             )
